<?php

namespace Teknisa\SM\Controller;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\SMTP as SMTPService;
use Teknisa\SM\Service\AlertaDesalocacaoRecursoAgendadoService;
use Teknisa\SM\Controller\BirtReport;
use Zeedhi\Framework\DTO\Row;

class AlertaDesalocacaoRecursoAgendadoController {
    
    protected $alertadesalocservice;
    protected $smtpservice;
    protected $birtController;
    
    public function __construct(AlertaDesalocacaoRecursoAgendadoService $alertadesalocservice, SMTPService $smtpservice, BirtReport $birtController){
        $this->alertadesalocservice = $alertadesalocservice;
        $this->smtpservice          = $smtpservice;
        $this->birtController       = $birtController;
    }
    
    public function alertaDesalocacaoRecurso(Request $request, Response $response){
        try {
            
            $nrorg = $request->getDataSet()->getRows()[0]["nrorg"];
            $dadosEmail = $this->alertadesalocservice->alertaDesalocacaoRecurso($nrorg);
            
            if(count($dadosEmail) > 0){
                $verificaEnvio = $this->enviaEmailAlertaDesalocacao($dadosEmail[0], $nrorg, $request, $response);
                if(!$verificaEnvio){
                    throw \Exception("Falha ao enviar E-mail.");
                }
            }
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Envia um email alertando que deve ser feita a desalocação de recurso
    **/
    public function enviaEmailAlertaDesalocacao($dadosEmail, $nrorg, $request, $response){
        $remetente = "no-reply@teknisa.com";
        $assunto   = "Relatório de Recursos Próximos ao Fim do Agendamento"; 
        
        $destinatario = $dadosEmail["DESTINATARIO"];
        $destinatario = str_replace(",", " ", $destinatario);
        $destinatarios = explode(" ", $destinatario);
        
        if(count($destinatarios) > 0){
            
            $paramsTemplate = array(
                "SUBJECT" => '',
	            "URL_LOGO_CLIENTE" => '',
	            "STRING_SUPERIOR_DIREITA" => '',
	            "TITLE_EMAIL" => 'RECURSOS PRÓXIMOS AO FIM DO AGENDAMENTO',
	            "FOOTER_TEMPLATE" => '<img src="https://www.teknisa.com/emails/img/rodape-sm.png" width="590" height="84" />',
                "TEXT_EMAIL" => '<p>Você está recebendo, anexo a este e-mail, o relatório com a listagem dos recursos que estão chegando na data final do agendamento.</p>
                                <p>&nbsp;</p>
                                <p>O intuito desta lista é auxiliar na organização e planejamento dos próximos agendamentos destes recursos, possibilitando que eles sejam feitos com antecedência.</p>
                                <p>&nbsp;</p>
                                <p>Obs: Caso seja necessário, o recurso pode ser desalocado manualmente através do registro de agendamento do recurso. Caso contrário, ao atingir a data final de agendamento (informada no relatório), o recurso será desalocado automaticamente e já estará dispónível para novos agendamentos.</p>'
            );
            
            $corpo = $this->buscatemplateCorpo($paramsTemplate);
            $anexo = $this->geraAnexoDesalocRecurso($nrorg, $dadosEmail["QTDIASDESALOCREC"], $request, $response);

            $tamanho = filesize($anexo);
            $tamanho = ($tamanho/1024)/1024;
            
            if ($tamanho > 15){
                unlink($anexo);
                throw new \Exception("Não foi possível enviar o E-mail. Não é permitido anexar documentos acima de 15Mb.");
            } else{
                $this->smtpservice->addAttachment($anexo);
            }
            
            $this->smtpservice->setFrom($remetente);
            $this->smtpservice->setFromName($remetente);
            $this->smtpservice->setSubject($assunto);
            
            $validaEnvioEmail = $this->smtpservice->sendMail($corpo, $destinatarios);
            
            if (is_file($anexo)) {
                unlink($anexo);
            }
            
            return $validaEnvioEmail;
        }
        
        return false;
    }
    
    
    /**
     * Gera o template teknisa com a mensagem do email
    **/
    public function buscatemplateCorpo(array $dadosSubstituiTemplate){
        $template = new \Teknisa\SM\Util\Mail\Templates\Teknisa\TemplateTeknisa();
        return $template->setValoresEmail($dadosSubstituiTemplate);
    }
    
    /**
     * Gera o anexo do relatório de recursos que devem ser desalocados
    **/
    public function geraAnexoDesalocRecurso($nrorg, $qtDiasAgendRec, $request, $response){
        $rowBirt = new Row();
        $rowBirt->set("__report_name", "recursos_final_agendamento");
        $rowBirt->set("P_QTDIASDESALOCREC", $qtDiasAgendRec);
        $rowBirt->set("NRORG", $nrorg);
        $requestBirt = new Request\Row($rowBirt, "POST", "/geraBirtReport", $request->getUserId());
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        if (isset($response->getMessages()[0])) {
            $urlRelatorio = $response->getMessages()[0]->getMessage();
            return $this->salvarRelatorioRecursos($urlRelatorio);
        }
    }
    
    /**
     * Salva o relatorio de recursos 
    **/
    public function salvarRelatorioRecursos($urlRelatorio){
        $microtime = microtime(true);
        $microtime = $microtime = str_replace(".", "_", $microtime);
        $nomeArquivo = $microtime."recursos_final_agendamento";
        
        $report = @file_get_contents($urlRelatorio);
        if ($report === false) {
            throw new \Exception('Servidor de emissão de relatórios indisponível. Contate o administrador de rede');
        }
    
        $reportPath = sys_get_temp_dir().DIRECTORY_SEPARATOR."$nomeArquivo.pdf";
        file_put_contents($reportPath, $report);
        unset($report);
        return $reportPath;
    }
    
}