<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Row;
use Teknisa\SM\Service\IntegracaoOmService;
use Teknisa\Libs\Util\SMTP as SMTPService;

class IntegracaoOmController extends SMCrud {
    
    protected $integracaoService;
    protected $birtController;
    protected $emailsAlertasMedPendentes;
    protected $qtDiasAntFimMedicao;
    protected $smtpservice;
    protected $geraParcelaMedicoesPendentes;
    
    public function __construct ($crudService, $environment, IntegracaoOmService $integracaoService, BirtReport $birtController, $emailsAlertasMedPendentes, $qtDiasAntFimMedicao, SMTPService $smtpservice, $geraParcelaMedicoesPendentes) {
        parent::__construct($crudService, $environment);
        $this->integracaoService            = $integracaoService;
        $this->birtController               = $birtController;
        $this->emailsAlertasMedPendentes    = $emailsAlertasMedPendentes;
        $this->qtDiasAntFimMedicao          = $qtDiasAntFimMedicao;
        $this->smtpservice                  = $smtpservice;
        $this->geraParcelaMedicoesPendentes = $geraParcelaMedicoesPendentes;
    }
    
    /**
     * Sincroniza os elementos do OM com o SM.
     * 
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function sincronizarElementos(Request $request, Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $codigo = $row[0]["code"];
            $produto     = $row["product"];
            $validacao = $this->integracaoService->validacomposicao($produto);
            $elementos   = $row["elements"];
            $solicitante = $row["requester"];
            $atualizarcomp = false;
            $nrorg      = $solicitante["organizationId"];
            $cdoperador = $solicitante["userId"];
            
            if ($validacao != null) {
                
                if($validacao[0]["IDELEMENTOPAI"] == true && $codigo === null){
                    throw new \Exception("ALERTA_POSSUI_ID_PAI");
                }
                
                if($codigo === true){
                    $atualizarcomp = true;
                }
            }

            $this->integracaoService->validaIntegracaoOm($produto, $elementos, $nrorg, $cdoperador, $atualizarcomp);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Realiza a integração com o OM para criar organização e liberar produtos/elementos.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function integracaoOM(Request $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nrorg = $row["NRORG"];
            $idutilizacomp = $row["IDUTILIZACOMP"];
            $periodoExtendido = $row["PERIODOEXTENDIDO"];
            $dtFinVigContr = $row["DTFINVIGCONTR"];
            $nrorcamento = $row["NRORCAMENTO"];
            
            $this->integracaoService->integracaoOM($nrseqorcamento, $nrorg, $idutilizacomp, $periodoExtendido, $dtFinVigContr, $nrorcamento);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Realiza a apuração automática das medições de faturamento.
     */
    public function apurarMedicoesAutomatico($nrorg) {
        if (!is_numeric($nrorg)) $nrorg = 20;
        try {
            $dadosAnteriores = $this->integracaoService->alteraStatusExecucaoScript($nrorg, 'I', null, null);
            
            $this->integracaoService->apurarMedicoesAutomatico($nrorg);
            $this->validarMedicoesSemColetaProxADataFinal($nrorg);
            
            if ($this->geraParcelaMedicoesPendentes) {
                $teveMedicoes = $this->integracaoService->apurarMedicoesQuantidadeContratual($nrorg);
                if ($teveMedicoes) {
                    try {
                        $this->enviarRelatorioMedicoesCalculadasQtdeContratual($nrorg);
                    }
                    catch (\Exception $e) {
                        $msg = "Erro ao gerar relatório de medições calculadas pela quantidade contratual: " . $e->getMessage();
                        $this->integracaoService->alteraStatusExecucaoScript($nrorg, 'E', $dadosAnteriores, $msg);
                        echo $msg . "\n";
                    }
                }
            }  

            $this->integracaoService->alteraStatusExecucaoScript($nrorg, 'F', $dadosAnteriores, null); 
        }
        catch (\Exception $e) {
            $msg = "Erro na apuração automática de medições: " . $e->getMessage();
            $this->integracaoService->alteraStatusExecucaoScript($nrorg, 'E', $dadosAnteriores, $msg);
        }
    }
    
    /**
     * Valida as medições próximas da data final que ainda não foram coletadas
    **/
    public function validarMedicoesSemColetaProxADataFinal($nrorg) {
        try {
            if (is_numeric($this->qtDiasAntFimMedicao)) {
                echo "\n Buscando as medições sem coleta realizada cuja data final de competência termina em até " . $this->qtDiasAntFimMedicao . " dias... \n";
                $qtdMedicoes = $this->integracaoService->getCountMedicoesSemColetas($nrorg, $this->qtDiasAntFimMedicao);
                if ($qtdMedicoes > 0) {
                    echo "Foram encontradas " . $qtdMedicoes . " medições. \n";
                    $this->integracaoService->gravaErrosColetaOM($nrorg, $this->qtDiasAntFimMedicao);
                    try {
                        echo "Gerando relatório de medições sem coleta... \n";
                        $this->enviarRelatorioMedicoesSemColeta($nrorg);
                    }
                    catch (\Exception $e) {
                        echo "Falha ao gerar o relatório de medições sem coleta: " . $e->getMessage() . "\n";
                    }
                }
                else {
                    echo "Não foi encontrada nenhuma medição. \n";
                }
            }
        }
        catch (\Exception $e) {
            echo "Erro ao gerar log de medições sem coleta: " . $e->getMessage() . "\n";
            throw $e;
        }
    }

    /**
     * envia o relatorio medicoes_nao_calculadas
    **/
    protected function enviarRelatorioMedicoesSemColeta($nrorg) {
        $remetente = "no-reply@teknisa.com";
        $assunto   = "Relatório de Medições sem Coletas"; 
        
        $destinatario  = $this->emailsAlertasMedPendentes;
        $destinatario  = str_replace(", ", " ", $destinatario);
        $destinatario  = str_replace(",", " ", $destinatario);
        $destinatarios = explode(" ", $destinatario);
        
        if(count($destinatarios) > 0) {
            $paramsTemplate = array(
                "SUBJECT"                 => '',
	            "URL_LOGO_CLIENTE"        => '',
	            "STRING_SUPERIOR_DIREITA" => '',
	            "TITLE_EMAIL"             => 'Medições sem Coletas',
	            "FOOTER_TEMPLATE"         => '<img src="https://www.teknisa.com/emails/img/rodape-sm.png" width="590" height="84" />',
                "TEXT_EMAIL"              => '<p>Você está recebendo, anexo a este e-mail, o relatório das medições que chegarão ao fim de sua competência em até '. $this->qtDiasAntFimMedicao .' dias e não possuem coleta realizada.</p>
                                              <p>&nbsp;</p>
                                              <p>Caso necessário, regularize suas informações e calcule-as pela tela "Medição de Faturamento"</p>'
            );
            
            $corpo = $this->getTemplateEmailTeknisa($paramsTemplate);
            $anexo = $this->gerarRelatorioMedicoesNaoColetadas($nrorg);

            $tamanho = filesize($anexo);
            $tamanho = ($tamanho/1024)/1024;
            
            if ($tamanho <= 15) {
                $this->smtpservice->addAttachment($anexo);
            } else {
                unlink($anexo);
                throw new \Exception("Não foi possível enviar o E-mail. Não é permitido anexar documentos acima de 15Mb.");
            }
            
            $this->smtpservice->setFrom($remetente);
            $this->smtpservice->setFromName($remetente);
            $this->smtpservice->setSubject($assunto);
            
            $validaEnvioEmail = $this->smtpservice->sendMail($corpo, $destinatarios);
            
            if (is_file($anexo))
                unlink($anexo);
            
            return $validaEnvioEmail;
        }
        
        return false;
    }
    
    /**
     * envia o relatorio medicoes_calculadas_qtde_contratual
    **/
    protected function enviarRelatorioMedicoesCalculadasQtdeContratual($nrorg) {
        $remetente = "no-reply@teknisa.com";
        $assunto   = "Relatório de Medições Calculadas pela Quantidade Contratual"; 
        
        $destinatario  = $this->emailsAlertasMedPendentes;
        $destinatario  = str_replace(", ", " ", $destinatario);
        $destinatario  = str_replace(",", " ", $destinatario);
        $destinatarios = explode(" ", $destinatario);
        
        if(count($destinatarios) > 0) {
            $paramsTemplate = array(
                "SUBJECT"                 => '',
	            "URL_LOGO_CLIENTE"        => '',
	            "STRING_SUPERIOR_DIREITA" => '',
	            "TITLE_EMAIL"             => 'Medições Calculadas pela Quantidade Contratual',
	            "FOOTER_TEMPLATE"         => '<img src="https://www.teknisa.com/emails/img/rodape-sm.png" width="590" height="84" />',
                "TEXT_EMAIL"              => '<p>Você está recebendo em anexo o relatório dos contratos que tiveram parcelas geradas utilizando a quantidade mínima contratual. As medições automáticas para esses contratos não foram realizadas antes do fechamento do período e deverão ser verificadas para que passem a ser realizadas a partir do próximo período.</p>'
            );
            
            $corpo = $this->getTemplateEmailTeknisa($paramsTemplate);
            $anexo = $this->gerarRelatorioMedicoesCalculadasQtdeContratual($nrorg);

            $tamanho = filesize($anexo);
            $tamanho = ($tamanho/1024)/1024;
            
            if ($tamanho <= 15) {
                $this->smtpservice->addAttachment($anexo);
            } else {
                unlink($anexo);
                throw new \Exception("Não foi possível enviar o E-mail. Não é permitido anexar documentos acima de 15Mb.");
            }
            
            $this->smtpservice->setFrom($remetente);
            $this->smtpservice->setFromName($remetente);
            $this->smtpservice->setSubject($assunto);
            
            $validaEnvioEmail = $this->smtpservice->sendMail($corpo, $destinatarios);
            
            if (is_file($anexo))
                unlink($anexo);
            
            return $validaEnvioEmail;
        }
        
        return false;
    }
    
    /**
     * Gera o template teknisa com a mensagem do email
    **/
    protected function getTemplateEmailTeknisa(array $dadosSubstituiTemplate){
        $template = new \Teknisa\SM\Util\Mail\Templates\Teknisa\TemplateTeknisa();
        return $template->setValoresEmail($dadosSubstituiTemplate);
    }
    
    /**
     * Gera o anexo do relatório medições sem coleta
    **/
    protected function gerarRelatorioMedicoesNaoColetadas($nrorg) {
        $rowBirt = new Row();
        $rowBirt->set("__report_name", "medicoes_nao_calculadas");
        $rowBirt->set("QTDIASANTEFIM", $this->qtDiasAntFimMedicao);
        $rowBirt->set("NRORG", $nrorg);
        $rowBirt->set("isUrl", true);
        
        $requestBirt = new Request\Row($rowBirt, "POST", "/geraBirtReport", null);
        $response    = new Response();
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        if (isset($response->getMessages()[0])) {
            $urlRelatorio = $response->getMessages()[0]->getMessage();
            return $this->salvarRelatorioRecursos($urlRelatorio, "medicoes_nao_calculadas");
        }
    }
    
    protected function gerarRelatorioMedicoesCalculadasQtdeContratual($nrorg) {
        $rowBirt = new Row();
        $rowBirt->set("__report_name", "medicoes_calculadas_qtde_contratual");
        $rowBirt->set("P_NRORG", $nrorg);
        $rowBirt->set("isUrl", true);

        $requestBirt = new Request\Row($rowBirt, "POST", "/geraBirtReport", null);
        $response    = new Response();
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        if (isset($response->getMessages()[0])) {
            $urlRelatorio = $response->getMessages()[0]->getMessage();
            return $this->salvarRelatorioRecursos($urlRelatorio, "medicoes_calculadas_qtde_contratual");
        }
    }
    
    /**
     * Salva o relatorio de medições sem coleta 
    **/
    protected function salvarRelatorioRecursos($urlRelatorio, $nameRelatorio){
        $microtime   = microtime(true);
        $microtime   = $microtime = str_replace(".", "_", $microtime);
        $nomeArquivo = $microtime . "_" . $nameRelatorio;
        
        $report = @file_get_contents($urlRelatorio);
        if ($report === false)
            throw new \Exception('Servidor de emissão de relatórios indisponível. Contate o administrador de rede.');
    
        $reportPath = sys_get_temp_dir() . DIRECTORY_SEPARATOR . "$nomeArquivo.pdf";
        file_put_contents($reportPath, $report);
        
        unset($report);
        return $reportPath;
    }
}

?>