<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class SmAssinaturaDigitalFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function getTokenCertsign (array $params) {
        $sql = "SELECT DSTOKENCERTSIGN 
                  FROM PARAM_CENTR_ASSINATURA
                 WHERE NRORG = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /*Busca o sequêncial do contrato ou proposta*/
    public function getSeqPropCont(array $params) {
        $sql = "SELECT DOC.NRSEQCONTDOC AS DOCUMENTO, DOC.NRSEQCONTRATO AS SEQUENCIAL, CONT.NRCONTRATOFCT AS CODIGO, 'CONTRATO' AS TIPO, DOC.NMARQUIVO, MODELO.DSMODELODOCORCA
                  FROM FCT_CONTRATODOC DOC
                  JOIN FCT_CONTRATO CONT
                    ON DOC.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                  LEFT JOIN FCT_MODELODOC MODELO
                    ON DOC.NRSEQMODDOCFCT = MODELO.NRSEQMODDOCFCT
                 WHERE DOC.IDDOCUMCERTSIGN = :IDDOCUMCERTSIGN
                   AND (DOC.IDSITUACAOCONTRATO = 'D' OR DOC.IDSITUACAOCONTRATO IS NULL)
                 UNION ALL
                SELECT DOC.NRSEQORCADOC AS DOCUMENTO, DOC.NRSEQORCAMENTO AS SEQUENCIAL, ORC.NRORCAMENTO AS CODIGO, 'PROPOSTA' AS TIPO, DOC.NMARQUIVO, MODELO.DSMODELODOCORCA
                  FROM FCT_ORCAMENTODOC DOC
                  JOIN FCT_ORCAMENTO ORC
                    ON DOC.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                  LEFT JOIN FCT_MODELODOC MODELO
                    ON DOC.NRSEQMODDOCFCT = MODELO.NRSEQMODDOCFCT
                 WHERE DOC.IDDOCUMCERTSIGN = :IDDOCUMCERTSIGN
                 AND EXISTS (SELECT 1
                               FROM FCT_ORCAMENTODOCSITU
                              WHERE NRSEQORCADOC = DOC.NRSEQORCADOC
                                AND IDSITUACAOORCAMENTO = 'CONCLUIDO')";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /*Atualiza situação das assinaturas da proposta*/
    public function updateAssinaProposta (array $params) {
        $sql = "UPDATE FCT_ASSINATDIGITALPROP
                   SET IDSITUACAO = :IDSITUACAO,
                       DSJUSTIFICARECUSA = :DSJUSTIFICARECUSA,
                       DTULTATU = SYSDATE
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    /*Atualiza situação das assinaturas do contrato*/
    public function updateAssinaContrato (array $params) {
        $sql = "UPDATE FCT_ASSINATDIGITALCONTR
                   SET IDSITUACAO = :IDSITUACAO,
                       DSJUSTIFICARECUSA = :DSJUSTIFICARECUSA,
                       DTULTATU = SYSDATE
                 WHERE NRASSDIGITALCONTR = :NRASSDIGITALCONTR
                   AND NRSEQCONTRATO = :NRSEQCONTRATO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    /* Recebe o sequencial de assinatura da proposta */
    public function recebeAssinaturaOrca (array $params){
        $sql = "SELECT NRASSDIGITALORCA 
                 FROM FCT_ASSINATDIGITALPROP
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRCPFSIGNATARIO = :NRCPFSIGNATARIO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura do contrato */
    public function recebeAssinaturaContr(array $params) {
        $sql = "SELECT NRASSDIGITALCONTR 
                 FROM FCT_ASSINATDIGITALCONTR
                WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                  AND NRCPFSIGNATARIO = :NRCPFSIGNATARIO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    
    /*Atualiza situação dos documentos das assinaturas da proposta*/
    public function updateAssinaDocProposta(array $params) {
        $sql = "UPDATE FCT_DOCASSINATPROP
                   SET IDSITUACAO        = :IDSITUACAO,
                       DSJUSTIFICARECUSA = :DSJUSTIFICARECUSA,
                       DTULTATU          = SYSDATE
                 WHERE NRSEQORCADOC     = :NRSEQORCADOC
                   AND NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO   = :NRSEQORCAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    /*Atualiza situação dos documentos das assinaturas do contrato */
    public function updateAssinaDocContrato (array $params) {
        $sql = "UPDATE FCT_DOCASSINATCONTR
                   SET IDSITUACAO        = :IDSITUACAO,
                       DSJUSTIFICARECUSA = :DSJUSTIFICARECUSA,
                       DTULTATU          = SYSDATE
                 WHERE NRSEQCONTDOC     = :NRSEQCONTDOC
                  AND NRASSDIGITALCONTR = :NRASSDIGITALCONTR
                  AND NRSEQCONTRATO     = :NRSEQCONTRATO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }
    
    /* Recebe situação e justificativa de recusa dos documentos de assinatura da proposta */
    public function recebeDadosAssDocProp (array $params){
    
        $sql = "SELECT IDSITUACAO, DSJUSTIFICARECUSA
                  FROM FCT_DOCASSINATPROP
                 WHERE NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND NRSEQORCAMENTO = :NRSEQORCAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
     /* Recebe situação e justificativa de recusa dos documentos de assinatura do contrato */
    public function recebeDadosAssDocContr (array $params){
    
        $sql = "SELECT IDSITUACAO, DSJUSTIFICARECUSA
                  FROM FCT_DOCASSINATCONTR
                 WHERE NRASSDIGITALCONTR = :NRASSDIGITALCONTR
                   AND NRSEQCONTRATO = :NRSEQCONTRATO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura da proposta */
    public function recebeDadosAssinaturaOrcaMail(array $params) {
        $sql = "SELECT ASS.NRASSDIGITALORCA,
                       ASS.IDTESTEMUNHA,
                       ASS.NRORDENACAO,
                       ASS.DSJUSTIFICARECUSA,
                       ASS.DSEMAILNOTSIGNENV,
                       ASS.DSEMAILNOTIFOUTROS,
                       DECODE(ASS.NRPARCNEGOCIO, NULL, INTER.NMINTERLOCUTOR, PARC.NMPRINCIPALPARC) AS NMSIGNATARIO
                  FROM FCT_ASSINATDIGITALPROP ASS
                  LEFT JOIN PARCNEGOCIO PARC
                    ON ASS.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                  LEFT JOIN CRM_INTERLOCUTOR INTER
                    ON ASS.NRSEQINTERLOCUTOR = INTER.NRSEQINTERLOCUTOR
                 WHERE ASS.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                   AND ASS.NRCPFSIGNATARIO = :NRCPFSIGNATARIO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura do contrato */
    public function recebeDadosAssinaturaContrMail (array $params){
        $sql = "SELECT ASS.NRASSDIGITALCONTR,
                       ASS.IDTESTEMUNHA,
                       ASS.NRORDENACAO,
                       ASS.DSJUSTIFICARECUSA,
                       ASS.DSEMAILNOTSIGNENV,
                       ASS.DSEMAILNOTIFOUTROS,
                       DECODE(ASS.NRPARCNEGOCIO, NULL, INTER.NMINTERLOCUTOR, PARC.NMPRINCIPALPARC) AS NMSIGNATARIO
                  FROM FCT_ASSINATDIGITALCONTR ASS
                  LEFT JOIN PARCNEGOCIO PARC
                    ON ASS.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                  LEFT JOIN CRM_INTERLOCUTOR INTER
                    ON ASS.NRSEQINTERLOCUTOR = INTER.NRSEQINTERLOCUTOR
                 WHERE ASS.NRSEQCONTRATO   = :NRSEQCONTRATO
                   AND ASS.NRCPFSIGNATARIO = :NRCPFSIGNATARIO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura da proposta */
    public function recebeEmailOperadoresOrca (array $params){
    
        $sql = "SELECT OPERADOR.DSEMAILOPER
                  FROM FCT_RELACOPERNOTIFASSELE
                  JOIN OPERADOR 
                    ON OPERADOR.CDOPERADOR = FCT_RELACOPERNOTIFASSELE.CDOPERADOR
                 WHERE FCT_RELACOPERNOTIFASSELE.NRASSDIGITALORCA = :NRASSDIGITALORCA
                   AND OPERADOR.DSEMAILOPER IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura do contrato */
    public function recebeEmailOperadoresCont(array $params) {
        $sql = "SELECT OPERADOR.DSEMAILOPER
                  FROM FCT_RELACOPERNOTIFASSELE
                  JOIN OPERADOR 
                    ON OPERADOR.CDOPERADOR = FCT_RELACOPERNOTIFASSELE.CDOPERADOR
                 WHERE FCT_RELACOPERNOTIFASSELE.NRASSDIGITALCONTR = :NRASSDIGITALCONTR
                   AND OPERADOR.DSEMAILOPER IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura do contrato */
    public function recebeEmailOperadorRespOrca(array $params) {
        $sql = "SELECT OPERADOR.DSEMAILOPER
                  FROM FCT_ORCAMENTO
                  JOIN OPERADOR 
                    ON OPERADOR.CDOPERADOR = FCT_ORCAMENTO.CDOPERRESPONSAVEL
                 WHERE FCT_ORCAMENTO.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND OPERADOR.DSEMAILOPER IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Recebe o sequencial de assinatura do contrato */
    public function recebeEmailOperadorRespCont(array $params) {
        $sql = "SELECT OPERADOR.DSEMAILOPER
                  FROM FCT_CONTRATO
                  JOIN OPERADOR 
                    ON OPERADOR.CDOPERADOR = FCT_CONTRATO.CDOPERRESPONSAVEL
                 WHERE FCT_CONTRATO.NRSEQCONTRATO = :NRSEQCONTRATO
                   AND OPERADOR.DSEMAILOPER IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os emails dos signatários da proposta informada.
     */
    public function recebeEmailSignatariosProposta(array $params, $filtro = null) {
        $sql = "SELECT EMAIL
                  FROM (
                    SELECT DISTINCT TRIM(NVL(INTER.EMAILINTERLOC, NVL(CP.CDCOMUNICAPARC, OP.DSEMAILOPER))) AS EMAIL
                      FROM FCT_ASSINATDIGITALPROP ASS
                      LEFT JOIN COMUNICAPARC CP
                        ON ASS.NRPARCNEGOCIO = CP.NRPARCNEGOCIO
                       AND ASS.NRORG         = CP.NRORG
                       AND CP.CDFORMACOMU    = '05'
                      LEFT JOIN CRM_INTERLOCUTOR INTER
                        ON ASS.NRSEQINTERLOCUTOR = INTER.NRSEQINTERLOCUTOR
                       AND ASS.NRORG             = INTER.NRORG
                      LEFT JOIN OPERADOR OP
                        ON ASS.NRPARCNEGOCIO  = OP.NRPARCNEGOCIO
                       AND ASS.NRORG          = OP.NRORG
                     WHERE ASS.NRSEQORCAMENTO = :NRSEQORCAMENTO
                       AND ASS.NRORG          = :NRORG
                       $filtro
                 )
                 WHERE EMAIL IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function recebeEmailVendedoresProposta(array $params) {
        $sql = "SELECT DISTINCT TRIM(CP.CDCOMUNICAPARC) AS EMAIL
                  FROM FCT_VENDEDORORCA VO
                  JOIN COMUNICAPARC CP
                    ON VO.NRPARCVENDEDOR = CP.NRPARCNEGOCIO
                   AND VO.NRORG          = CP.NRORG
                 WHERE VO.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND VO.NRORG          = :NRORG
                   AND CP.CDFORMACOMU    = '05'
                   AND CP.CDCOMUNICAPARC IS NOT NULL";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function recebeEmailInterlocutorProposta(array $params) {
        $sql = "SELECT DISTINCT TRIM(INTER.EMAILINTERLOC) AS EMAILINTERLOC
                  FROM FCT_ORCAMENTO ORC
                  JOIN CRM_INTERLOCUTOR INTER
                    ON ORC.NRSEQINTERLOCUTOR = INTER.NRSEQINTERLOCUTOR
                   AND ORC.NRORG             = INTER.NRORG
                 WHERE ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND ORC.NRORG          = :NRORG
                   AND INTER.EMAILINTERLOC IS NOT NULL";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    /**
     * Retorna os emails dos signatários do contrato informado.
     */
    public function recebeEmailSignatariosContrato(array $params, $filtro = null) {
        $sql = "SELECT EMAIL
                  FROM (
                    SELECT DISTINCT TRIM(NVL(CP.CDCOMUNICAPARC, OP.DSEMAILOPER)) AS EMAIL
                      FROM FCT_ASSINATDIGITALCONTR ASS
                      LEFT JOIN COMUNICAPARC CP
                        ON ASS.NRPARCNEGOCIO = CP.NRPARCNEGOCIO
                       AND ASS.NRORG         = CP.NRORG
                       AND CP.CDFORMACOMU    = '05'
                      LEFT JOIN OPERADOR OP
                        ON ASS.NRPARCNEGOCIO  = OP.NRPARCNEGOCIO
                       AND ASS.NRORG          = OP.NRORG
                     WHERE ASS.NRSEQCONTRATO = :NRSEQCONTRATO
                       AND ASS.NRORG         = :NRORG
                       $filtro
                 )
                 WHERE EMAIL IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function inserirLogAssinaturaDigital(array $params) {
        $sql = "INSERT INTO FCT_LOGASSINATDIGITAL (
                    NRLOGASSINATDIGITAL,    NRSEQORCAMENTO,     NRSEQCONTRATO,
                    IDDOCUMCERTSIGN,        NRCPFSIGNATARIO,    NRSEQORCADOC,
                    NRSEQCONTDOC,           DSERRO,             NRORG,
                    DTINCLUSAO
                )
                VALUES (
                    :NRLOGASSINATDIGITAL,   :NRSEQORCAMENTO,    :NRSEQCONTRATO,
                    :IDDOCUMCERTSIGN,       :NRCPFSIGNATARIO,   :NRSEQORCADOC,
                    :NRSEQCONTDOC,          :DSERRO,            :NRORG,
                    SYSDATE
                )";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }

/* DADOS CDA - PROVISÓRIO */

    public function getServPrestDoc (array $params) {
      $sql = "SELECT NRSERVPRESTDOC
                FROM SERVPRESTDOC
              WHERE IDDOCUMCERTSIGN = :IDDOCUMCERTSIGN";
      
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      return $std->fetchAll();
    }

    public function recebeSignatarioCDA (array $params) {
      $sql = "SELECT NRSERVPRESTDOCASSINA
                FROM SERVPRESTDOCASSINA
              WHERE NRSERVPRESTDOC = :NRSERVPRESTDOC
                AND NRCPFSIGNATARIO = :NRCPFSIGNATARIO";
      
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      return $std->fetchAll();
    }

    public function updateServPrestDoc(array $params, $idSituacao) {
      if ($idSituacao == 'P'){

          $sql = "UPDATE SERVPRESTDOC
                    SET IDSITASSINATURA = :IDSITASSINATURA
                  WHERE NRSERVPRESTDOC = :NRSERVPRESTDOC";   
      }
      else if ($idSituacao == 'A'){
          $sql = "UPDATE SERVPRESTDOC
                    SET IDSITASSINATURA = :IDSITASSINATURA,
                        DTASSINATURA = SYSDATE 
                  WHERE NRSERVPRESTDOC = :NRSERVPRESTDOC";
          
      } 
      else {
          $sql = "UPDATE SERVPRESTDOC
                    SET IDSITASSINATURA = :IDSITASSINATURA,
                        DTASSINATURA = SYSDATE,
                        DSJUSTIFRECUSAASSINAT = :DSJUSTIFRECUSAASSINAT  
                  WHERE NRSERVPRESTDOC = :NRSERVPRESTDOC";
      }

      $ps = $this->connection->prepare($sql);
      $ps->execute($params);
    }

    public function recebeDadosSignatariosCDA (array $params) {
      $sql = "SELECT NRSERVPRESTDOCASSINA, IDSITASSINATURA, DSJUSTIFRECUSAASSINAT
                FROM SERVPRESTDOCASSINA
              WHERE NRSERVPRESTDOC = :NRSERVPRESTDOC";
      
      $std = $this->connection->prepare($sql);
      $std->execute($params);
      return $std->fetchAll();
    }

    public function  updateAssinaDocCDA (array $params) {
      $sql = "UPDATE SERVPRESTDOCASSINA
                SET IDSITASSINATURA = :IDSITASSINATURA,
                    DTASSINATURA = SYSDATE,
                    DSJUSTIFRECUSAASSINAT = :DSJUSTIFRECUSAASSINAT  
              WHERE NRSERVPRESTDOC = :NRSERVPRESTDOC
                AND NRSERVPRESTDOCASSINA = :NRSERVPRESTDOCASSINA";
      
      $std = $this->connection->prepare($sql);
      $std->execute($params);
    }

    public function inserirLogAssinaturaDigitalCDA(array $params) {
      $sql = "INSERT INTO SERVPRESTDOCLOGASSDIG (
                  NRSERVPRESTDOCLOGASSDIG,    NRSERVPRESTDOC,     IDDOCUMCERTSIGN,
                  NRCPFSIGNATARIO,       DSERRO,             NRORG,
                  DTINCLUSAO
              )
              VALUES (
                  :NRSERVPRESTDOCLOGASSDIG,   :NRSERVPRESTDOC,    :IDDOCUMCERTSIGN,
                  :NRCPFSIGNATARIO,   :DSERRO,            :NRORG,
                  SYSDATE
              )";
      
      $ps = $this->connection->prepare($sql);
      $ps->execute($params);
    }

}
?>